import { exists, run } from "./utils.ts";
import * as path from "https://deno.land/std@0.214.0/path/mod.ts";

export class NginxController {
	private proxy_target_domain;
	private enabled_conf_dir;

	constructor(proxy_target_domain: string, enabled_conf_dir: string) {
		this.proxy_target_domain = proxy_target_domain;
		this.enabled_conf_dir = enabled_conf_dir;
	}

	/*
server {
    server_name barnulf.net;
    location / {
        proxy_pass http://barnulf.net:8051;
    }

    listen [::]:443 ssl ipv6only=on; # managed by Certbot
    listen 443 ssl; # managed by Certbot
    ssl_certificate /etc/letsencrypt/live/barnulf.net/fullchain.pem; # managed by Certbot
    ssl_certificate_key /etc/letsencrypt/live/barnulf.net/privkey.pem; # managed by Certbot

}
	*/

	public async add_proxy(domain: string, port: number, conf_dir: string, tls: boolean) {
		const conf_file_path = await this.set_http_config(domain, port, conf_dir);
		await this.reload();
		if (tls) {
			await this.gen_cert(domain);
			await this.set_https_config(domain, port, conf_dir);
			await this.reload();
		}
		return conf_file_path;
	}

	private async set_https_config(domain: string, port: number, conf_dir: string) {
		const conf_file_content = `
server {
	listen 80;
	listen [::]:80;
	server_name ${domain};
	return 301 https://$host$request_uri;
}

server {
	listen 443 ssl;
	listen [::]:443 ssl;
	server_name ${domain};
	ssl_certificate     /etc/letsencrypt/live/${domain}/fullchain.pem; # managed by Certbot
	ssl_certificate_key /etc/letsencrypt/live/${domain}/privkey.pem;   # managed by Certbot
	include             /etc/letsencrypt/options-ssl-nginx.conf; # managed by Certbot
	ssl_dhparam         /etc/letsencrypt/ssl-dhparams.pem;       # managed by Certbot
	location / {
		proxy_pass http://${this.proxy_target_domain}:${port};
	}
}
`;
		const conf_file_path = path.join(conf_dir, domain + ".conf");
		const enabled_conf_file_path = path.join(this.enabled_conf_dir, domain + ".conf");
		await Deno.writeTextFile(conf_file_path, conf_file_content);
		if (await exists(enabled_conf_file_path)) await Deno.remove(enabled_conf_file_path);
		await run("ln", "-s", await Deno.realPath(conf_file_path), enabled_conf_file_path);
		return conf_file_path;
	}

	private async set_http_config(domain: string, port: number, conf_dir: string) {
		const conf_file_content = `
server {
	listen 80;
	listen [::]:80;
	server_name ${domain};
	location / {
		proxy_pass http://${this.proxy_target_domain}:${port};
	}
}
`;
		const conf_file_path = path.join(conf_dir, domain + ".conf");
		const enabled_conf_file_path = path.join(this.enabled_conf_dir, domain + ".conf");
		await Deno.writeTextFile(conf_file_path, conf_file_content);
		if (await exists(enabled_conf_file_path)) await Deno.remove(enabled_conf_file_path);
		await run("ln", "-s", await Deno.realPath(conf_file_path), enabled_conf_file_path);
		return conf_file_path;
	}

	private async gen_cert(domain: string) {
		await run(
			"certbot",
			"--nginx",
			...["-d", domain],
			"--non-interactive",
			"--agree-tos",
			...["--email", "matthieu@imagevo.fr"],
		);
	}

	public async remove_proxy(domain: string, conf_dir: string) {
		const conf_file_path = path.join(conf_dir, domain + ".conf");
		const enabled_conf_file_path = path.join(this.enabled_conf_dir, domain + ".conf");
		await Deno.remove(enabled_conf_file_path, { recursive: true });
		await Deno.remove(conf_file_path, { recursive: true });
		await this.reload();
	}

	public async *read_all_in_dir(conf_dir: string) {
		for await (const { name } of Deno.readDir(conf_dir)) {
			const [domain, rest] = name.split(".conf");
			if (rest !== "") continue;
			yield domain;
		}
	}

	private async reload() {
		await run("systemctl", "restart", "nginx");
	}
}

/*
server {
    if ($host = barnulf.net) {
        return 301 https://$host$request_uri;
    } # managed by Certbot


    listen 80;
    listen [::]:80;
    server_name barnulf.net;
    return 404; # managed by Certbot


}

*/
